#ifndef NODE_ARRAY_H_
#define NODE_ARRAY_H_

#include <graph/Graph.h>
#include <sarray/Range.h>

#include <string>
#include <map>

class SubSetIndex;
class SArray;

/**
 * A NodeArray is a container class for nodes with a dimension
 * attribute.  The array can be tiled with nodes using the set()
 * function.  These nodes can be retrieved using the get() function.
 * Arbitrary subsets of the NodeArray can be returned with the
 * getSubSet() function.
 *
 * Node arrays are created when Nodes are declared at the start
 * of a JAGS model description.
 */
class NodeArray {
  Range _range;
  Graph _graph;
  Node **_node_pointers;
  long *_offsets;
  std::map<Range, Node *, ltrange> _set_nodes;
  std::map<Range, Node *, ltrange> _generated_nodes;
  std::string _name;

  /* Forbid copying */
  NodeArray(NodeArray const &orig);
  NodeArray &operator=(NodeArray const &rhs);

public:
  /**
   * Constructor. Creates a NodeArray of the given dimension
   */
  NodeArray(Index const &dim);
  ~NodeArray();
  /**
   * Sets the name of the node array
   */
  void setName(std::string const &name);
  /**
   * Returns the name of the node array
   */
  std::string const &name() const;
  /**
   * Sets the given range of the NodeArray to refer to the node.
   * The dimension of the node must conform with the given range.
   * The given range must not overlap with any previously set range.
   *
   * The node is added to an internal graph. If the node does not yet
   * have a name, then its name is set based on the NodeArray name
   * and the given range (e.g. x[1:10,1])
   *
   * @exception runtime_error
   */
  void set(Node *node, Range const &range);
  /**
   * Determines whether a given range overlaps with ranges used in
   * previous calls to the set() function. If this is true, then the
   * range cannot be used in a future call to set.
   */
  bool rangeContainsSetNodes(Range const &range) const;
  /**
   * Returns a node corresponding to the given range.  The range must
   * have been used in a previous call to set, otherwise a NULL
   * pointer is returned.
   */
  Node* get(Range const &range) const;
  /**
   * Returns an arbitrary subset of the NodeArray.  If the given
   * range has any gaps (i.e. it has not been filled with previous
   * calls to set) these will be tiled with constant nodes. Then
   * an aggregate node will be created, if necessary, to represent
   * the subset.  Any nodes created will be added to the graph of
   * the NodeArray.
   */
  Node* getSubset(Range const &range);
  /**
   * Sets the values of the stochastic nodes in the array. If the
   * observed parameter is true then the nodes must be either
   * stochastic or constant. If it is false, then they must be
   * stochastic.
   *
   * @param value SArray containing values to be used. The value
   * vector of the SArray may contain missing values.  If so, then the
   * part of the value vector corresponding to each set node must be
   * either completely missing (in which no action is taken) or
   * contain no missing values.
   * @param observed. Flag indicating whether the nodes are
   * considered as observed or not, i.e. whether the values are
   * data or simply initial values.
   */
  void setValue(SArray const &value, bool observed);
  /**
   * Gets the values of the nodes in the array. Deterministic nodes
   * are ignored, unless they are constant nodes and the observed 
   * parameter is true.
   *
   * @param value SArray to which values should be written.
   * @param observed. Flag indicating whether to write values of
   * observed nodes (i.e. the data) or unobserved nodes (i.e. current
   * values of the parameters).
   */
  void getValue(SArray &value, bool observed) const;
  /**
   * Returns the range of the  NodeArray
   */
  Range const &range() const;
  /**
   * Returns the graph associated with the NodeArray.  This graph
   * contains all the nodes that were either used in a call to
   * set, or created during a call to getSubset.
   */
  Graph const &graph() const;
};


#endif /* NODE_ARRAY_H */
